%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GECAD SSCI 2025 Competition: Evolutionary Computation in the Energy Domain: 2025 Edition of the Risk-based Energy Scheduling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ALGORITMH: GIRCEDUMDA
% Grouped Importance RCEDUMDA
% The algorithm, based on RCEDUMDA, quantifies the optimization potential of each group using inter-generational fitness difference, achieving adaptive allocation of the search space.
%% Developers: 
% Main author: Qiongfang Liu, ShenZhen University, China
% Supervisor: Qingling Zhu, ShenZhen University, China
%% References:
% The codes of RCEDUMDA available http://www.gecad.isep.ipp.pt/ERM-competitions/2023-2/
% Authors: Ansel Y. Rodríguez González (Team Leader) 
%          Ángel Díaz Pacheco           
%          Ramón Aranda               
%          Miguel Á. Álvarez-Carmona      
%          Yoan Martínez López                 
%          Julio Madera
function [Fit_and_p,FVr_bestmemit, fitMaxVector] = ...
    GIRCEDUMDA(vRCEDUMDAparameters,caseStudyData,otherParameters,low_habitat_limit,up_habitat_limit,No_evals)

I_NP         = vRCEDUMDAparameters.I_NP;
I_D          = numel(up_habitat_limit);

I_lowPop     = vRCEDUMDAparameters.I_lowPop;

FVr_minbound = low_habitat_limit;
FVr_maxbound = up_habitat_limit;

I_itermax    = vRCEDUMDAparameters.I_itermax;

I_indByCell         = vRCEDUMDAparameters.I_indByCell;
I_StructureWidth    = vRCEDUMDAparameters.I_StructureWidth;
I_StructureHeight   = vRCEDUMDAparameters.I_StructureHeight;
I_NeighborhoodType  = vRCEDUMDAparameters.I_NeighborhoodType;
I_NeighborhoodParam = vRCEDUMDAparameters.I_NeighborhoodParam;

I_SelectionMethod   = vRCEDUMDAparameters.I_SelectionMethod;
I_Selected          = vRCEDUMDAparameters.I_Selected; 

I_elitismParam = vRCEDUMDAparameters.I_elitismParam;

I_CodeSize     = vRCEDUMDAparameters.I_CodeSize;

I_EstimationParamsMethod = vRCEDUMDAparameters.I_EstimationParamsMethod; 
I_EstimationParamsParam  = vRCEDUMDAparameters.I_EstimationParamsParam;

I_EstimationStructureMethod = vRCEDUMDAparameters.I_EstimationStructureMethod; 
I_EstimationStructureParam  = vRCEDUMDAparameters.I_EstimationStructureParam ;

I_ScalerMethod = vRCEDUMDAparameters.I_ScalerMethod; 
I_ScalerParam  = vRCEDUMDAparameters.I_ScalerParam;

I_SamplingMethod = vRCEDUMDAparameters.I_SamplingMethod; 
I_SamplingParam  = vRCEDUMDAparameters.I_SamplingParam;

I_maxFits     = vRCEDUMDAparameters.I_maxFits; 
I_repeatEvals = vRCEDUMDAparameters.I_repeatEvals; 

global I_evaluationProxy                 ;
I_evaluationProxy                 = vRCEDUMDAparameters.I_evaluationProxy;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fnc          =  otherParameters.fnc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fitMaxVector = nan(1,I_itermax);

[~,solSize] = size(FVr_minbound); 

global FullStoreSol;  
global FullStoreVal;  
global FullStoreCount;

if (I_repeatEvals == 0)
    FullStoreSol   = zeros(I_NP * I_itermax,solSize);
    FullStoreVal   = zeros(1,I_NP * I_itermax);
    FullStoreCount = 0;
end 
if (I_repeatEvals == 2)
    FullStoreSol   = zeros(I_elitismParam,solSize);
    FullStoreVal   = zeros(1,I_elitismParam);
    FullStoreCount = 0;
end 


global EvaluationProxyInitTrainGenCount;  
global EvaluationProxyEvalGenCount;  
global EvaluationProxyTrainGenCount;
global EvaluationProxyState;

EvaluationProxyInitTrainGenCount = 0;  
EvaluationProxyEvalGenCount      = 0;  
EvaluationProxyTrainGenCount     = 0;
EvaluationProxyState             = 0;

global Train_X;
global Train_Y;
global Train_Count;
global ModelTrained;

[~,featuresSize] = size(FVr_minbound);

Train_X = nan(vRCEDUMDAparameters.maxEvals, featuresSize);
Train_Y = nan(vRCEDUMDAparameters.maxEvals, 1); 
Train_Count = 0;

 global group_importance;
 group_importance=zeros(1,I_D);
 delta_OF=zeros(1,5);
 global group_sizes;
 group_sizes=[0,42,500,25,2,1];

gen = 1; 
evalCount = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----FM_pop is a matrix of size I_NPx(I_D+1). It will be initialized------
%----with random values between the min and max values of the-------------
%----parameters-----------------------------------------------------------
% FLC modification - vectorization
minPositionsMatrix=repmat(FVr_minbound,I_NP,1);
maxPositionsMatrix=repmat(FVr_maxbound,I_NP,1);

%code inicialized
codes = initializeCodes(FVr_minbound,FVr_maxbound,I_CodeSize);

% generate initial population.
rand('state',otherParameters.iRuns) %Guarantee same initial population

FM_pop = genpop(I_NP,I_D,minPositionsMatrix,maxPositionsMatrix);

for iiii = 1:I_lowPop
    FM_pop(iiii,:) = FVr_minbound;
end    


S_val = my_feval(fnc,FM_pop,caseStudyData, otherParameters,I_repeatEvals,No_evals);   

[~,I_best_index] = min(S_val); 
 
FVr_bestmemit = FM_pop(I_best_index,:); 
fitMaxVector(1,gen)   = S_val(I_best_index);

enCodedPop = enCodePop(FM_pop,codes,FVr_minbound,FVr_maxbound);
newEnCodedPop = enCodedPop;

[~,pbidx_all] = sort(S_val);
pbidx = pbidx_all(1:I_elitismParam);    
pbpop = enCodedPop(pbidx,:);

 if (I_repeatEvals == 2)       
     FullStoreSol   = FM_pop(pbidx,:);
     FullStoreVal   = S_val(pbidx);
     FullStoreCount = I_elitismParam; 
 end

eCount = 0;

while (eCount<I_itermax) && (evalCount < vRCEDUMDAparameters.maxEvals-I_NP)    

    for cellIndexH = 1:I_StructureHeight
        for cellIndexW = 1:I_StructureWidth

            NeighborsIndexes          = getNeighborsIndexes(cellIndexH,cellIndexW,I_StructureHeight,I_StructureWidth,I_NeighborhoodType,I_NeighborhoodParam,I_indByCell);
            SelectedIndividualIndexes = getSelectedIndividualIndexes(NeighborsIndexes,S_val,I_SelectionMethod,I_Selected);        
            
            [Px,Pxy] = getEstimadedParams(enCodedPop,S_val,SelectedIndividualIndexes,codes,I_EstimationParamsMethod,I_EstimationParamsParam);
           
            [Dxgy,EdgesList,EdgesCount] = getEstimadedStructure(Px,Pxy,I_EstimationStructureMethod,I_EstimationStructureParam);
            
            [Px,Pxy] = getScaledParams(Px,Pxy,length(SelectedIndividualIndexes),I_ScalerMethod,I_ScalerParam);
             
            cellLinearIndex = I_StructureWidth * (cellIndexH-1) + cellIndexW;
            CellIndividualIndexes = getCellIndividualLinearIndexes(cellLinearIndex, I_indByCell); 
                    
            NewCellPop = sampleNewIndividualsFromModel(I_indByCell,Px,Pxy,Dxgy,EdgesList,EdgesCount,I_SamplingMethod,I_SamplingParam);
          
            for ii = 1: I_indByCell            
                newEnCodedPop(CellIndividualIndexes(ii),:) = NewCellPop(ii,:);
            end
              
        end
    end
    
    enCodedPop = newEnCodedPop;
    
    for ii = 1: I_elitismParam            
        enCodedPop(pbidx(ii),:) = pbpop(ii,:); 
    end
    
    
    deCodedPop = deCodePop(enCodedPop,codes);
   
    %Evaluation of new Pop
    previousEvaluationProxyState = EvaluationProxyState;
    previousEvaluationProxyInitTrainGenCount = EvaluationProxyInitTrainGenCount;
    S_val = my_feval(fnc,deCodedPop,caseStudyData, otherParameters,I_repeatEvals,No_evals);
        
    [~,pbidx_all] = sort(S_val);
    pbidx = pbidx_all(1:I_elitismParam);    
    pbpop = enCodedPop(pbidx,:);
       
    if (I_repeatEvals == 2)       
        FullStoreSol   = deCodedPop(pbidx,:);
        FullStoreVal   = S_val(pbidx);
        FullStoreCount = I_elitismParam; 
    end   

    avgV   = mean(S_val);
    worstV = max(S_val);
        
    %% update best results
    [S_bestval,I_best_index] = min(S_val);
    FVr_bestmemit = deCodedPop(I_best_index,:);
           
    fprintf('Best Worst Avg Fitness value: %f %f %f %f\n',S_val(I_best_index),S_bestval,avgV,worstV);
    fprintf('Generation: %d\n',gen);
 
    gen=gen+1; 
    
    fitMaxVector(1,gen) = S_val(I_best_index);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Divide into 5 groups, each running one generation separately to calculate the fitness difference.
    if gen>=2 && gen<=6
        delta_OF(1,gen-1)=fitMaxVector(1,gen-1)-fitMaxVector(1,gen);
        fprintf('delta_OF: %d\n',delta_OF);
    end
    if gen==7
        priority=1:5;
        %priority ranking
        for i = 1:4
            for j = i+1:5
                if delta_OF(i) < delta_OF(j) || ...
                   (delta_OF(i) == delta_OF(j) && group_sizes(i+1) > group_sizes(j+1))
                    temp_OF = delta_OF(i);
                    delta_OF(i) = delta_OF(j);
                    delta_OF(j) = temp_OF;
                    
                    temp_idx = priority(i);
                    priority(i) = priority(j);
                    priority(j) = temp_idx;
                end
            end
        end
        fprintf('priority: %d\n',priority);

        for i=1:5
            x=priority(i);
            for j=1:24
                if i==5
                    group_importance(1+sum(group_sizes(1:x))+570*(j-1):570*(j-1)+sum(group_sizes(1:x+1)))=0.1;
                else
                   group_importance(1+sum(group_sizes(1:x))+570*(j-1):570*(j-1)+sum(group_sizes(1:x+1)))=1-(0.01*(i-1));
                end
            end
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    eCount = eCount + 1;

end
Fit_and_p=[fitMaxVector(1,gen) 0]; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plotResults(fitMaxVector);
hold on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
function codes = initializeCodes(FVr_minbound,FVr_maxbound,codeSize)
    ws = length(FVr_minbound);
    codes = nan(codeSize,ws);
    for jjj=1:ws
       range = FVr_maxbound(jjj) - FVr_minbound(jjj);
       for iii=1:codeSize 
          codes(iii,jjj) = FVr_minbound(jjj) + ( range / (codeSize - 1.0) * (iii-1.0));    
       end
       
       if (codes(iii,jjj)>FVr_maxbound(jjj))
           codes(codeSize,jjj) = FVr_maxbound(jjj);
       end
    end
end    

function enCodedPop = enCodePop(pop,codes,FVr_minbound,FVr_maxbound)
    [hs,ws] = size(pop);
    enCodedPop = nan(hs,ws);
    for iii=1:hs
       enCodedPop(iii,:) = enCode(pop(iii,:),codes,FVr_minbound,FVr_maxbound);
    end
end    
    
function enCoded = enCode(indiv,codes,FVr_minbound,FVr_maxbound)
    [hs,ws] = size(codes);
    enCoded = nan(1,ws);
    for jjj=1:ws
       if (FVr_minbound(jjj)==FVr_maxbound(jjj)) 
            enCoded(1,jjj)=1;
       else           
            range = (FVr_maxbound(jjj) - FVr_minbound(jjj))/hs;
            sum = FVr_minbound(jjj) + range;
            iii = 1;
            while (indiv(jjj) > sum) && (iii<hs) 
                  iii = iii + 1;
                  sum = sum + range;
            end
            enCoded(1,jjj)=iii;
       end
    end
end    
    
function deCodedPop = deCodePop(enCodedPop,codes)
    [hs,ws] = size(enCodedPop);
    deCodedPop = nan(hs,ws);
    for iii=1:hs
       deCodedPop(iii,:) = deCode(enCodedPop(iii,:),codes);
    end
end    
    
function indiv = deCode(enCoded,codes)
   [~,ws] = size(codes);
    indiv = nan(1,ws);   
    for jjj=1:ws
        pos = enCoded(1,jjj);
        v   = codes(pos,jjj);
        indiv(1,jjj)=v;
    end
end    

function NeighborsIndexes=getNeighborsIndexes(~,cellIndexW,~,I_StructureWidth,I_NeighborhoodType,I_NeighborhoodParam,I_indByCell)
    switch I_NeighborhoodType
        case 1 % left-right 2k-1
            NeighborsCellIndexes = getNeighborsCellIndexes_LeftRight(cellIndexW,I_StructureWidth,I_NeighborhoodParam);   
    end
    NeighborsIndexes = getCellIndividualLinearIndexes(NeighborsCellIndexes(1), I_indByCell);
    [~,ws] = size(NeighborsCellIndexes);
    for index = 2:ws
       M = getCellIndividualLinearIndexes(NeighborsCellIndexes(index), I_indByCell);
       NeighborsIndexes = [NeighborsIndexes M];
    end
end    

function SelectedIndividualIndexes = getSelectedIndividualIndexes(NeighborsIndexes,S_val,I_SelectionMethod,I_Selected)
    
    switch I_SelectionMethod
        case 1 % Trunc Selection
            SelectedIndividualIndexes = getTruncSelectedIndividualIndexes(NeighborsIndexes,S_val,I_Selected);   
        otherwise
            SelectedIndividualIndexes = [];
    end
end    


 
function [Px,Pxy] = getEstimadedParams(enCodedPop,S_val,SelectedIndividualIndexes,codes,I_EstimationParamsMethod,~)
    switch I_EstimationParamsMethod
        case 1    % 1 Bivariated
            %%%% Removed method. It is not relevant for RCED-UMDA
            % [Px,Pxy] = getEstimadedParamsBivariated(enCodedPop,SelectedIndividualIndexes,codes);
        otherwise % 0 Univariated
            [Px,Pxy] = getEstimadedParamsUnivariated(enCodedPop,S_val,SelectedIndividualIndexes,codes);    
    end
end    
        
function [Px,Pxy] = getEstimadedParamsUnivariated(FM_pop,~,SelectedIndividualIndexes,codes)
    parents = FM_pop(SelectedIndividualIndexes, :);
    [chs,cws] = size(codes); 
    [phs,~] = size(parents);
    Pxy = nan;
    
    Px   = zeros(chs,cws);
    
    ivx = 1;
    while(ivx<=cws)
      
       id = 1;
       while (id <=phs)
          codeX = parents(id,ivx);
          Px(codeX,ivx) = Px(codeX,ivx)+1;              
          id = id + 1; 
       end
        
       ivx = ivx + 1; 
    end

end 
    
function [Dxgy,EdgesList,EdgesCount] = getEstimadedStructure(~,~,I_EstimationStructureMethod,~)
    switch I_EstimationStructureMethod
        case 1
            %%%% Removed method. It is not relevant for RCED-UMDA
            % [Dxgy,EdgesList,EdgesCount] = getEstimadedStructure_OrderPrecedenceMaxDependency(Px,Pxy,I_EstimationStructureParam); 
        otherwise
            Dxgy = zeros(1,1);
            EdgesList  = nan(1,1);
            EdgesCount = zeros(1,1);           
    end
end       
   
function [Px,Pxy] = getScaledParams(Px,Pxy,n,I_ScalerMethod,I_ScalerParam)
    switch I_ScalerMethod
        case 1 
           %%%% Removed method. It is not relevant for RCED-UMDA
           % [Px,Pxy] = getScaledParamsWithMinValue(Px,Pxy,n,I_ScalerParam);          
        case 2   
           [Px,Pxy] = getScaledParamsAddingValue(Px,Pxy,n,I_ScalerParam);
           
        otherwise    
                    
    end
end      
    
function [Px,Pxy] = getScaledParamsAddingValue(Px,Pxy,n,I_ScalerParam)
    [chs,~] = size(Px);  

    group_matrix=zeros(1,I_D);
    if gen>=1 && gen<=5
        group_matrix=generate_group_matrix(gen,I_D);
    end
  
    singleAdd = I_ScalerParam * n / chs;
    doubleAdd = I_ScalerParam * n / (chs*chs);

    if gen<6
        Px  = singleAdd* group_matrix + Px;% group_matrix = 1 for the current group and 0 otherwise
    else
        Px  = singleAdd* group_importance + Px;
    end
    Pxy = doubleAdd + Pxy;
end    

function group_matrix=generate_group_matrix(gen,I_D)
    group_matrix=zeros(1,I_D);
    offset = 0;

    if gen>=2 && gen<=6
        for i = 1:(gen-1)
            offset = offset + group_sizes(i);
        end
        group_size = group_sizes(gen);
        for j=1:24
            start_index = 1+offset+(j-1)*570;
            end_index =start_index+group_size-1;
            group_matrix(start_index:end_index)=1;
        end
    end
end
    
function NewCellPop = sampleNewIndividualsFromModel(I_indByCell,Px,~,~,~,~,I_SamplingMethod,~)
   switch I_SamplingMethod
        case 1
           %%%% Removed method. It is not relevant for RCED-UMDA
           % NewCellPop = sampleNewIndividualsFrom_OrderPrecedenceMaxDependency(I_indByCell,Px,Pxy,Dxgy,EdgesList,EdgesCount);
        otherwise    
           NewCellPop = sampleNewIndividualsFromModelWithoutDependencies(I_indByCell,Px);         
   end
end      
    
function NewCellPop = sampleNewIndividualsFromModelWithoutDependencies(I_indByCell,Px)
    [~,vws] = size(Px);
     NewCellPop = rand(I_indByCell,vws);
    
     for ic = 1: I_indByCell            
        for iv = 1: vws
           NewCellPop(ic,iv) = rulete(NewCellPop(ic,iv), Px(:,iv));
        end
     end
end     
             
function value = rulete(rv, Counters) 
    len = length(Counters); 
    
    total = 0;
    for value=1:len
        total = total + Counters(value);
    end
    
    rv = rv * total;
    
    sum = 0;
    for value=1:len
        sum = sum + Counters(value);
        if (rv<=sum+eps(sum)) 
            return;
        end
    end
    value = len;
end    
   
function NeighborsIndexes = getNeighborsCellIndexes_LeftRight(cellIndexW,I_StructureWidth,I_NeighborhoodParam)
    NeighborsIndexes = nan(1,2*I_NeighborhoodParam+1,'single');
    for index = 1:2*I_NeighborhoodParam+1
        NeighborsIndexes(index) = cellIndexW - I_NeighborhoodParam - 1 + index;
        if (NeighborsIndexes(index) < 1)
           NeighborsIndexes(index) = I_StructureWidth + NeighborsIndexes(index);
        end
        if (NeighborsIndexes(index) > I_StructureWidth)
           NeighborsIndexes(index) = NeighborsIndexes(index) - I_StructureWidth;
        end  
    end
end    

function CellIndividualLinearIndexes = getCellIndividualLinearIndexes(cellIndex,I_indByCell)
    CellIndividualLinearIndexes = nan(1,I_indByCell,'single');
    base = (cellIndex-1) * I_indByCell;
    for index = 1:I_indByCell
       CellIndividualLinearIndexes(index) = base + index;
    end
end    

function SelectedIndividualIndexes = getTruncSelectedIndividualIndexes(NeighborsIndexes,S_val,I_Selected)
    [~,niS] = size(NeighborsIndexes);
    NeighborsVal = nan(1,niS,'single');
    for index = 1:niS
        NeighborsVal(index)=S_val(NeighborsIndexes(index));
    end
    [~,I] = sort(NeighborsVal);

    SelectedIndividualIndexes = nan(1,I_Selected);
    for index = 1:I_Selected
        SelectedIndividualIndexes(index)=NeighborsIndexes(I(index));
    end
end    
 
function plotResults(Results)
    [~,ws] = size(Results);
    X=1:1:ws;
    Y=Results;
    plot(X,Y); 
    grid on
end    

% VECTORIZED THE CODE INSTEAD OF USING FOR
function pop=genpop(a,b,lowMatrix,upMatrix)
    pop=unifrnd(lowMatrix,upMatrix,a,b);
end

function [Val, evalCount] = my_feval(fnc,deCodedPop,caseStudyData, otherParameters, I_repeatEvals,No_evals)    
    switch I_repeatEvals
        case 0
           %%%% Removed method. It is not relevant for RCED-UMDA
           % [Val, evalCount] = my_feval_withStore(fnc,deCodedPop,caseStudyData, otherParameters, evalCount);
        case 2
           Val = my_feval_withStoreForElitism(fnc,deCodedPop,caseStudyData, otherParameters,No_evals); 
        otherwise    
            [s,~] = size(deCodedPop);    
            evalCount = evalCount + s;
            [Val, ~] = feval(fnc,deCodedPop,caseStudyData, otherParameters,No_evals);
            fprintf('evalCount: %d\n',evalCount);       
    end
end  

function Val = my_feval_withStoreForElitism(fnc,deCodedPop,caseStudyData, otherParameters,No_evals)    
     [cs,ws] = size(deCodedPop);
     Val    = nan(1,cs);     
     
     count = 0;
     xPop  = nan(cs,ws);
     xIndex = nan(cs,1);
     
     for iii=1:cs
        index = searchSolution(deCodedPop(iii,:),ws);
        if index > 0
            Val(iii) = FullStoreVal(index);           
        else
           count = count + 1;
           xIndex(count) =  iii;
           xPop(count,:) = deCodedPop(iii,:);                   
        end
     end
     
     xxPop = xPop(1:count,:);
     xxIndex = xIndex(1:count);
     
     auxVal = feval_proxy(fnc,xxPop,caseStudyData, otherParameters,No_evals);
     
     Val(xxIndex) = auxVal(:);
     
     fprintf('evalCount: %d\n',evalCount);
end

function newPop = reduceToDecisionFeatures(pop)
   newPop = pop(:,decisionFeatures); 
end

function Val = feval_proxy(fnc,pop,caseStudyData, otherParameters,No_evals) 
    [s1,s2] = size(pop);
    
    if (I_evaluationProxy == 0)
        %% evaluo f 
        %% fprintf('Eval ...\n');
        Val    = nan(1,s1); 
        for index=1:s1
           %fprintf('index: %d\n',index); 
           [auxVal, ~] = feval(fnc,pop(index,:),caseStudyData, otherParameters,No_evals);
           Val(index) = auxVal;
           evalCount = evalCount + 1; 
           
        end
    else
       %%%% Removed block. It is not relevant for this submission.
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    end   
end
     
function index = searchSolution(sol, ws)
     index = 0; 
    
     for iii=1:FullStoreCount
                 
         flag = 1;
         for jjj=1:ws
            if FullStoreSol(iii,jjj)~=sol(jjj)
              flag = 0;
              break;
            end
         end
         
         if flag == 1
            index = iii;
            fprintf('*');
            break; 
         end         
     end
end             

end